﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Webus.Analysis;
using System.Text.RegularExpressions;

namespace OutlookSearcher
{
    public class MailAnalyzer : IAnalyzer
    {
        public ITokenStream GetTokenStream(string value)
        {
            return new WordTokenStream(value);
        }

        public ITokenStream GetTokenStream(Webus.Documents.Field field)
        {
            return this.GetTokenStream(field.Value.ToString());
        }
    }

    public class WordTokenStream : ITokenStream
    {
        HashSet<string> stops = new HashSet<string>(new string[] {
            "的",
"一",
"不",
"在",
"人",
"有",
"是",
"为",
"以",
"于",
"上",
"他",
"而",
"后",
"之",
"来",
"及",
"了",
"因",
"下",
"可",
"到",
"由",
"这",
"与",
"也",
"此",
"但",
"并",
"个",
"其",
"已",
"无",
"小",
"我",
"们",
"起",
"最",
"再",
"今",
"去",
"好",
"只",
"又",
"或",
"很",
"亦",
"某",
"把",
"那",
"你",
"乃",
"它",
"吧",
"被",
"比",
"别",
"趁",
"当",
"从",
"到",
"得",
"打"
        });
        Queue<Token> Buffer = new Queue<Token>();
        public WordTokenStream(string value)
        {
            MatchCollection mc = Regex.Matches(value, @"[\u4e00-\u9fa5]|[a-zA-Z0-9\._]+");
            foreach (Match m in mc)
            {
                if (stops.Contains(m.Value.ToLower()) == false)
                {
                    Buffer.Enqueue(new Token(m.Value.ToLower(), m.Index, m.Length));
                }
            }
        }
        public void Close()
        {
            this.Dispose();
        }

        public Token Current
        {
            get;
            private set;
        }

        public bool Next()
        {
            if (Buffer.Count > 0)
            {
                this.Current = Buffer.Dequeue();
                return true;
            }
            else
            {
                return false;
            }
        }

        public void Dispose()
        {
            this.Buffer.Clear();
        }
    }
}
